/*		 ITI-FG  Kernel module free public release
 *
 *		     Copyright (C) 1996-2000 GOM mbH.
 *	         Rebenring 33, 38106 Braunschweig, GERMANY
 *		            All rights reserved
 *
 * $Id: ampcvReg.c,v 7.0 2000/10/25 09:07:57 cvs Exp $
 *
 */

/*
 * Defines:
 */

#if (DEBUG_ITIFG & 4)
#  define DEBUG_ITIFG_PLL
#endif
#ifdef DEBUG_ITIFG_PLL
#  define ICP_PRINTR(fmt,args...) iti_printra (fmt, ## args)
#else
#  define ICP_PRINTR(fmt,args...)
#endif

#define READ_HANDLER_BIT(subfield, reg_name) \
	inline u_char \
	pv_get_ ## subfield (itifg_t *iti) \
	{ \
	  pv_ ## reg_name ## _t tmp_val; \
	  tmp_val.all = iti_readw (&iti->pv_reg-> ## reg_name ## .all); \
	  return (tmp_val.b. ## subfield); \
        }
				 
#define READ_HANDLER_FIELD(subfield, reg_name) \
	inline u_char \
	pv_get_ ## subfield (itifg_t *iti) \
	{ \
	  pv_ ## reg_name ## _t tmp_val; \
	  tmp_val.all = iti_readw (&iti->pv_reg-> ## reg_name ## .all); \
	  return (tmp_val.b. ## subfield); \
        }
				 
#define WRITE_HANDLER_BIT(subfield, reg_name) \
	inline int \
	pv_set_ ## subfield (itifg_t *iti, u_char bit_value) \
	{ \
	  pv_ ## reg_name ## _t tmp_val; \
	  tmp_val.all = iti_readw (&iti->pv_reg-> ## reg_name ## .all); \
          tmp_val.b. ## subfield = bit_value; \
	  iti_writew (tmp_val.all, &iti->pv_reg-> ## reg_name ## .all); \
	  return (OK); \
        }
				 
/*
 * Includes:
 */

#include "itifgOS.h"	/* os specific stuff */
#include "itifgExt.h"	/* offical driver interface */
#include "itifgReg.h"	/* register definitions ICR Regs */
#include "icpciReg.h"	/* register definitions IC-PCI */
#include "amvsReg.h"	/* register definitions AM-VS */
#include "amdigReg.h"	/* register definitions AM-DIG */
#include "ampcvReg.h"	/* register definitions PCVision */
#include "pcdigReg.h"	/* register definitions PC-DIG */
#include "itifgIntD.h"	/* driver internals */
#include "itifgIntF.h"	/* internal functions */

/*
 * Globals:
 */

#if (DEBUG_ITIFG & 132)
const char  *pv_reg_name[] = { "modid  ",
			       "error  ",
			       "error  ",
			       "error  ",
			       "lutaddr",
			       "lutdata",
			       "adcdata",
			       "error  ",
			       "htotal ",
			       "hsyncpw",
			       "hequipw",
			       "hserrpw",
			       "vtotal ",
			       "vsyncpw",
			       "vgstart",
			       "vgwidth",
			       "hoffset",
			       "hactive",
			       "voffset",
			       "vactive",
			       "strbdly",
			       "frstdly",
			       "con1   ",
			       "con2   ",
			       "trigcon",
			       "error  ",
			       "p_clamp",
			       "error  ",
			       "error  ",
			       "error  ",
			       "error  ",
			       "error  ",
			       /* ---- */
			       "pllreg0",
			       "pllreg1",
			       "pllreg2",
			       "pllreg3",
			       "pllreg4",
			       "pllreg5",
			       "pllreg6",
			       "pllreg7",
			       /* ---- */
			       "negref "
			       "posref ",
};
#endif

/*
 * Prototypes:
 */

static u_short
pv_read_pllreg (itifg_t*, u_short);
static int
pv_write_pllreg (itifg_t*, u_char, u_short);
static int
pv_set_lutaddr (itifg_t*, u_char);

/*
 * Read/Write Functions for Clock Generator
 * for Details on Operation see Data Sheet ICS1522
 * from Integrated Circuit Systems Inc.
 */

static u_short
pv_read_pllreg (itifg_t *iti, u_short addr) {

  int count;

  u_char  dummy = 0;
  u_short tmp_val = 0;

  if (addr > PV_PLLADDR_MASK - 1) {
    iti_printwa ("error in pv_read_pllreg - addr %d > %d \n",
		 addr, PV_PLLADDR_MASK - 1);
    return (0);
  }

  /* write address */
  ITI_PRINT2("    READ  I: ");
  iti_writew (iti_readw (&iti->pv_reg->con2.all) & ~1,
	      &iti->pv_reg->con2.all); /* PV_PLL_PROG */
  ICP_PRINTR("%c ", (iti_readw (&iti->pv_reg->con2.all) & 1) ?
	      'H' : 'L');
  iti_writew (PV_PLL_READ, &iti->pv_reg->pllprg.all);
  ICP_PRINTR("%d ", PV_PLL_READ);
  for (count = 0; count < 14; count++) {
    dummy = (addr >> count) & 1;
    ICP_PRINTR("%d ", dummy);
    iti_writew (dummy, &iti->pv_reg->pllprg.all);
  }
  iti_writew (iti_readw (&iti->pv_reg->con2.all) | 1,
	      &iti->pv_reg->con2.all ); /* PLL_PLL_NORM */
  ICP_PRINTR("%c ", (PV_PLL_NORM) ? 'H' : 'L');
  iti_writew (0, &iti->pv_reg->pllprg.all);
  ICP_PRINTR("%d.\n", dummy);
  
  /* read value */
  ITI_PRINT2("    READ II: ");
  iti_writew (iti_readw (&iti->pv_reg->con2.all) & ~1,
	      &iti->pv_reg->con2.all); /* PV_PLL_PROG */
  ICP_PRINTR("%c ", (iti_readw (&iti->pv_reg->con2.all) & 1) ?
	     'H' : 'L');
  for (count = 0; count < 14; count++) {
    dummy = iti_readw (&iti->pv_reg->pllprg.all) & 1;
    ICP_PRINTR("%d ", dummy);
    tmp_val |= dummy << count;
  }
  dummy = iti_readw (&iti->pv_reg->pllprg.all) & 1;
  ICP_PRINTR("%d ", dummy);
  iti_writew (iti_readw (&iti->pv_reg->con2.all) | 1,
	      &iti->pv_reg->con2.all ); /* PLL_PLL_NORM */
  ICP_PRINTR("%c ", (PV_PLL_NORM) ? 'H' : 'L');
  iti_writew (0, &iti->pv_reg->pllprg.all);
  ICP_PRINTR("0\n");   
  
  if ((tmp_val & PV_PLLADDR_MASK) != addr) {
    iti_printwa ("error in pv_read_pllreg - addr %d != %d\n",
		 (tmp_val & PV_PLLADDR_MASK), addr);
    return (0);
  }

  return (tmp_val >> PV_PLLDATA_SHIFT);
}

static int
pv_write_pllreg (itifg_t *iti, u_char addr, u_short value) {

  int count;

  u_char  dummy;
  u_short tmp_val = 0;

  if (addr > PV_PLLADDR_MASK - 1) {
    iti_printwa ("error in pv_write_pllreg - addr %d > %d \n",
		 addr, PV_PLLADDR_MASK - 1);
    return (ERROR);
  }
  
  if (value & ~PV_PLLDATA_MASK) {
    iti_printwa ("error in pv_write_pllreg - value %d > %d \n",
		 addr, PV_PLLDATA_MASK);
    return (ERROR);
  }
  
  tmp_val = addr | ((value & PV_PLLDATA_MASK) << PV_PLLDATA_SHIFT);
  
  /* write combined address/data */
  ITI_PRINT2("    WRITE I: ");
  iti_writew (iti_readw (&iti->pv_reg->con2.all) & ~1,
	      &iti->pv_reg->con2.all); /* PV_PLL_PROG */
  ICP_PRINTR("%c ", (iti_readw (&iti->pv_reg->con2.all) & 1) ?
	      'H' : 'L');
  iti_writew (PV_PLL_WRITE, &iti->pv_reg->pllprg.all);  
  ICP_PRINTR("%d ", PV_PLL_WRITE);
  for (count = 0; count < 14; count++) {
    dummy = (tmp_val >> count) & 1;
    ICP_PRINTR("%d ", dummy);
    iti_writew (dummy, &iti->pv_reg->pllprg.all);
  }
  iti_writew (iti_readw (&iti->pv_reg->con2.all) | 1,
	      &iti->pv_reg->con2.all ); /* PLL_PLL_NORM */
  ICP_PRINTR("%c ", (iti_readw (&iti->pv_reg->con2.all) & 1) ?
	      'H' : 'L');
  iti_writew (0, &iti->pv_reg->pllprg.all);  
  ICP_PRINTR("0\n");
  
  return (OK);
}

/*
 * Special Read Handler for PCVision Registers
 */

READ_HANDLER_BIT(hsyncpol, vsyncpw)
READ_HANDLER_BIT(vsyncpol, vsyncpw)
READ_HANDLER_BIT(lenpol, strbdly)
READ_HANDLER_BIT(fenpol, strbdly)
READ_HANDLER_BIT(frstonv, frstdly)
READ_HANDLER_BIT(frstmd, frstdly)
READ_HANDLER_BIT(frstpol, frstdly)
READ_HANDLER_BIT(frstsz, frstdly)
READ_HANDLER_BIT(syncsel, con1);
READ_HANDLER_BIT(chsyncen, con1);
READ_HANDLER_BIT(vsyncen, con1);
READ_HANDLER_BIT(clksrc, con1);
READ_HANDLER_BIT(lpfsel, con1);
READ_HANDLER_BIT(vscan, con1);
READ_HANDLER_BIT(iscale, con1);
READ_HANDLER_BIT(vclkpol, con1);
READ_HANDLER_BIT(smode, con1);
READ_HANDLER_BIT(fldshft, con1);
READ_HANDLER_BIT(einpol, con2);
READ_HANDLER_BIT(ein0ien, con2);
READ_HANDLER_BIT(ein1ien, con2);
READ_HANDLER_BIT(ein0, con2);
READ_HANDLER_BIT(ein1, con2);
READ_HANDLER_BIT(eout0, con2);
READ_HANDLER_BIT(eout1, con2);
READ_HANDLER_BIT(sotint, con2);
READ_HANDLER_BIT(eotint, con2);
READ_HANDLER_BIT(ein0int, con2);
READ_HANDLER_BIT(ein1int, con2);
READ_HANDLER_BIT(trigen, trigcon)
READ_HANDLER_BIT(trigcy, trigcon)
READ_HANDLER_BIT(sotien, trigcon)
READ_HANDLER_BIT(eotien, trigcon)
READ_HANDLER_BIT(trigsrc, trigcon)
READ_HANDLER_BIT(trigpol, trigcon)
READ_HANDLER_BIT(strbmd, trigcon)
READ_HANDLER_BIT(strben, trigcon)
READ_HANDLER_BIT(strbpol, trigcon)
READ_HANDLER_BIT(skpfldmd, trigcon)
READ_HANDLER_BIT(pclmpol, p_clamp)
READ_HANDLER_BIT(clmpsrc, p_clamp)

READ_HANDLER_FIELD(rev, modid)
READ_HANDLER_FIELD(camsel, con1);

u_char
pv_get_lutdata (itifg_t *iti, struct pv_lut_t *get_lut) {
  
  int count;
  
  if (pv_set_lutaddr (iti, PV_LUTADDR_BASE) < 0) return (ERROR);
  
  for (count = PV_MIN_LUTDATA; count <= PV_MAX_LUTDATA; count++)
    get_lut->lutdata[count] = iti_readw (&iti->pv_reg->lutdata);
  
  return (OK);
}

u_char
pv_get_pref (itifg_t *iti) {

  if (pv_set_lutaddr (iti, PV_LUTADDR_PREF) < 0)
    iti_error (iti, "pv_set_lutaddr");

  return (iti_readw (&iti->pv_reg->adcdata));
}

u_char
pv_get_nref (itifg_t *iti) {

  if (pv_set_lutaddr (iti, PV_LUTADDR_NREF) < 0)
      iti_error (iti, "pv_set_lutaddr");
  
  return (iti_readw (&iti->pv_reg->adcdata));
}

u_short
pv_get_htotal (itifg_t *iti) {

  return ((iti_readw (&iti->pv_reg->htotal) << 1) + 1);
}

u_char
pv_get_hsyncpw (itifg_t *iti) {

  return (iti_readw (&iti->pv_reg->hsyncpw) + 1);
}  

u_char
pv_get_hequipw (itifg_t *iti) {

  return (iti_readw (&iti->pv_reg->hequipw) + 1);
}  

u_short
pv_get_hserrpw (itifg_t *iti) {

  return (iti_readw (&iti->pv_reg->hserrpw) + 1);
}  

u_short
pv_get_vtotal (itifg_t *iti) {
  
  return (iti_readw (&iti->pv_reg->vtotal) + 1);
}  

u_char
pv_get_vsyncpw (itifg_t *iti) {
  
  pv_vsyncpw_t tmp_val;

  tmp_val.all = iti_readw (&iti->pv_reg->vsyncpw.all);
  return (++tmp_val.b.vsyncpw);
}  

u_short
pv_get_vgstart (itifg_t *iti) {
  
  return (iti_readw (&iti->pv_reg->vgstart) + 1);
}  

u_char
pv_get_vgwidth (itifg_t *iti) {

  return (iti_readw (&iti->pv_reg->vgwidth) + 1);
}  

u_short
pv_get_hoffset (itifg_t *iti) {

  return ((iti_readw (&iti->pv_reg->hoffset) & PV_HOFFSET_MASK) + 1);
}  

u_short
pv_get_hactive (itifg_t *iti) {

  return ((iti_readw (&iti->pv_reg->hactive) & PV_HACTIVE_MASK) + 1);
}  

u_short
pv_get_voffset (itifg_t *iti) {

  return ((iti_readw (&iti->pv_reg->voffset) & PV_VOFFSET_MASK) + 1);
}  

u_short
pv_get_vactive (itifg_t *iti) {

  return ((iti_readw (&iti->pv_reg->vactive) & PV_VACTIVE_MASK) + 1);
}  

u_char
pv_get_strbdly (itifg_t *iti) {

  pv_strbdly_t tmp_val;

  tmp_val.all = iti_readw (&iti->pv_reg->strbdly.all);
  return (++tmp_val.b.strbdly);
}  

u_char
pv_get_froff (itifg_t *iti) {

  pv_frstdly_t tmp_val;

  tmp_val.all = iti_readw (&iti->pv_reg->frstdly.all);
  return (++tmp_val.b.froff);
}  

u_char
pv_get_ccnt (itifg_t *iti) {

  pv_p_clamp_t tmp_val;

  tmp_val.all = iti_readw (&iti->pv_reg->p_clamp.all);
  return (tmp_val.b.ccnt << 1);
}

u_short
pv_get_fdiv (itifg_t *iti) {

  return ((pv_read_pllreg (iti, PV_PLLREG0_ADDR) & PV_FBKDIV_MASK) + 1);
}

u_short
pv_get_rdiv (itifg_t *iti) {

  return ((pv_read_pllreg (iti, PV_PLLREG3_ADDR) & PV_REFDIV_MASK) + 1);
}

u_char
pv_get_vcogain (itifg_t *iti) {

  return (pv_read_pllreg (iti, PV_PLLREG4_ADDR) & PV_VCO_MASK);
}

u_char
pv_get_pda (itifg_t *iti) {

  return ((pv_read_pllreg (iti, PV_PLLREG5_ADDR) & PV_PDA_MASK) >>
	  PV_PDA_SHIFT);
}

u_char
pv_get_lcnt (itifg_t *iti) {

  return (pv_read_pllreg (iti, PV_PLLREG6_ADDR) & PV_LCNT_MASK);  
}

/*
 * Special Write Handler for PCVision Registers
 */

WRITE_HANDLER_BIT(hsyncpol, vsyncpw)
WRITE_HANDLER_BIT(vsyncpol, vsyncpw)
WRITE_HANDLER_BIT(frstonv, frstdly)
WRITE_HANDLER_BIT(frstmd, frstdly)
WRITE_HANDLER_BIT(frstpol, frstdly)
WRITE_HANDLER_BIT(frstsz, frstdly)
WRITE_HANDLER_BIT(syncsel, con1);
WRITE_HANDLER_BIT(chsyncen, con1);
WRITE_HANDLER_BIT(vsyncen, con1);
WRITE_HANDLER_BIT(clksrc, con1);
WRITE_HANDLER_BIT(lpfsel, con1);
WRITE_HANDLER_BIT(iscale, con1);
WRITE_HANDLER_BIT(smode, con1);
WRITE_HANDLER_BIT(fldshft, con1);
WRITE_HANDLER_BIT(einpol, con2);
WRITE_HANDLER_BIT(ein0ien, con2);
WRITE_HANDLER_BIT(ein1ien, con2);
WRITE_HANDLER_BIT(eout0, con2);
WRITE_HANDLER_BIT(eout1, con2);
WRITE_HANDLER_BIT(trigen, trigcon)
WRITE_HANDLER_BIT(sotien, trigcon)
WRITE_HANDLER_BIT(eotien, trigcon)
WRITE_HANDLER_BIT(trigsrc, trigcon)
WRITE_HANDLER_BIT(trigpol, trigcon)
WRITE_HANDLER_BIT(strbmd, trigcon)
WRITE_HANDLER_BIT(strbpol, trigcon)
WRITE_HANDLER_BIT(skpfldmd, trigcon)
WRITE_HANDLER_BIT(pclmpol, p_clamp)
WRITE_HANDLER_BIT(clmpsrc, p_clamp)

static int
pv_set_lutaddr (itifg_t *iti, u_char value) {

  switch (value) {
  case PV_LUTADDR_CONT:
    iti_writew (PV_LUTADDR_CONT, &iti->pv_reg->lutaddr);
    return (OK);
  case PV_LUTADDR_NREF:
    iti_writew (PV_LUTADDR_NREF, &iti->pv_reg->lutaddr);
    return (OK);
  case PV_LUTADDR_PREF:
    iti_writew (PV_LUTADDR_PREF, &iti->pv_reg->lutaddr);
    return (OK);
  }

  iti->errno = -EITI_NOSUPPORT;
  return (ERROR);
}

int
pv_set_lutdata (itifg_t *iti, struct pv_lut_t *set_lut) {

  int count;

  if (set_lut == NULL) {
    iti->errno = -EITI_POINTER;
    return (ERROR);
  }

  if (pv_set_lutaddr (iti, PV_LUTADDR_BASE) < 0) return (ERROR);

  for (count = PV_MIN_LUTDATA; count <= PV_MAX_LUTDATA; count++)
    iti_writew (set_lut->lutdata[count], &iti->pv_reg->lutdata);

  return (OK);
}

int
pv_set_pref (itifg_t *iti, u_char value) {

  if (pv_set_lutaddr (iti, PV_LUTADDR_PREF) < 0) return (ERROR);

  if (IS_PV_REV_A)
    iti_writew (value, &iti->pv_reg->adcdata);
  else
    iti_writew (value & PV_PREF_MASK, &iti->pv_reg->adcdata);
  
  return (OK);
}

int
pv_set_nref (itifg_t *iti, u_char value) {

  if (pv_set_lutaddr (iti, PV_LUTADDR_NREF) < 0) return (ERROR);
  
  if (IS_PV_REV_A)
    iti_writew (value, &iti->pv_reg->adcdata);
  else
    iti_writew (value & PV_NREF_MASK, &iti->pv_reg->adcdata);
  
  return (OK);
}

int
pv_set_htotal (itifg_t *iti, u_short value) {

  if (value < PV_MIN_HTOTAL || value > PV_MAX_HTOTAL) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }
  iti_writew (((value >> 1) - 1) & PV_HTOTAL_MASK, &iti->pv_reg->htotal);

  return (OK);
}

int
pv_set_hsyncpw (itifg_t *iti, u_char value) {

  if (value < PV_MIN_HSYNCPW || value > PV_MAX_HSYNCPW) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }
  iti_writew (--value & PV_HSYNCPW_MASK, &iti->pv_reg->hsyncpw);

  return (OK);
}  

int
pv_set_hequipw (itifg_t *iti, u_char value) {

  if (value < PV_MIN_HEQUIPW || value > PV_MAX_HEQUIPW) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }
  iti_writew (--value & PV_HEQUIPW_MASK, &iti->pv_reg->hequipw);

  return (OK);
}  

int
pv_set_hserrpw (itifg_t *iti, u_short value) {

  if (value < PV_MIN_HSERRPW || value > PV_MAX_HSERRPW) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }
  iti_writew (--value & PV_HSERRPW_MASK, &iti->pv_reg->hserrpw);

  return (OK);
}  

int
pv_set_vtotal (itifg_t *iti, u_short value) {
  
  if (value < PV_MIN_VTOTAL || value > PV_MAX_VTOTAL) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }
  iti_writew (--value & PV_VTOTAL_MASK, &iti->pv_reg->vtotal);
  
  return (OK);
}  

int
pv_set_vsyncpw (itifg_t *iti, u_char value) {

  pv_vsyncpw_t tmp_val;

  if (value < PV_MIN_VSYNCPW || value > PV_MAX_VSYNCPW) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }

  tmp_val.all = iti_readw (&iti->pv_reg->vsyncpw.all);
  tmp_val.b.vsyncpw = --value;
  iti_writew (tmp_val.all, &iti->pv_reg->vsyncpw.all);

  return (OK);
}  

int
pv_set_vgstart (itifg_t *iti, u_short value) {
  
  if (value < PV_MIN_VGSTART || value > PV_MAX_VGSTART) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }
  iti_writew (--value & PV_VGSTART_MASK, &iti->pv_reg->vgstart);

  return (OK);
}  

int
pv_set_vgwidth (itifg_t *iti, u_char value) {
  
  if (value < PV_MIN_VGWIDTH || value > PV_MAX_VGWIDTH) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }
  iti_writew (--value & PV_VGWIDTH_MASK, &iti->pv_reg->vgwidth);

  return (OK);
}  

int
pv_set_hoffset (itifg_t *iti, u_short value) {

  if (value < PV_MIN_HOFFSET || value > PV_MAX_HOFFSET) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }
  iti_writew (--value & PV_HOFFSET_MASK, &iti->pv_reg->hoffset);

  return (OK);
}  

int
pv_set_hactive (itifg_t *iti, u_short value) {

  if (value < PV_MIN_HACTIVE || value > PV_MAX_HACTIVE) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }
  iti_writew (--value & PV_HACTIVE_MASK, &iti->pv_reg->hactive);

  return (OK);
}  

int
pv_set_voffset (itifg_t *iti, u_short value) {

  if (value < PV_MIN_VOFFSET || value > PV_MAX_VOFFSET) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }
  iti_writew (--value & PV_VOFFSET_MASK, &iti->pv_reg->voffset);

  return (OK);
}  

int
pv_set_vactive (itifg_t *iti, u_short value) {

  if (IS_PV_REV_A) {
    if (value < PV_MIN_VACTIVE || value > PV_MAX_VACTIVEA) {
      iti->errno = -EITI_RANGE;
      return (ERROR);
    }
  } else {
    if (value < PV_MIN_VACTIVE || value > PV_MAX_VACTIVEB) {
      iti->errno = -EITI_RANGE;
      return (ERROR);
    }
  }
  iti_writew (--value & PV_VACTIVE_MASK, &iti->pv_reg->vactive);

  return (OK);
}  

int
pv_set_strbdly (itifg_t *iti, u_char value) {

  pv_strbdly_t tmp_val;

  if (value < PV_MIN_STRBDLY || value > PV_MAX_STRBDLY) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }
  
  tmp_val.all = iti_readw (&iti->pv_reg->strbdly.all);
  tmp_val.b.strbdly = --value & PV_STRBDLY_MASK;
  iti_writew (tmp_val.all, &iti->pv_reg->strbdly.all);

  return (OK);
}  

int
pv_set_lenpol (itifg_t *iti, u_char value) {

  pv_strbdly_t tmp_val;

  if (IS_PV_REV_A) {
    iti->errno = -EITI_NOSUPPORT;
    return (ERROR);
  }
  
  tmp_val.all = iti_readw (&iti->pv_reg->strbdly.all);
  tmp_val.b.lenpol = value;
  iti_writew (tmp_val.all, &iti->pv_reg->strbdly.all);

  return (OK);
}

int
pv_set_fenpol (itifg_t *iti, u_char value) {

  pv_strbdly_t tmp_val;
  
  if (IS_PV_REV_A) {
    iti->errno = -EITI_NOSUPPORT;
    return (ERROR);
  }
  
  tmp_val.all = iti_readw (&iti->pv_reg->strbdly.all);
  tmp_val.b.fenpol = value;
  iti_writew (tmp_val.all, &iti->pv_reg->strbdly.all);

  return (OK);
}

int
pv_set_froff (itifg_t *iti, u_char value) {

  pv_frstdly_t tmp_val;

  if (IS_PV_REV_A) {
    if (value < PV_MIN_FROFF || value > PV_MAX_FROFFA) {
      iti->errno = -EITI_RANGE;
      return (ERROR);
    }
  } else {
    if (value < PV_MIN_FROFF || value > PV_MAX_FROFFB) {
      iti->errno = -EITI_RANGE;
      return (ERROR);
    }
  }

  tmp_val.all = iti_readw (&iti->pv_reg->frstdly.all);
  tmp_val.b.froff = --value & PV_FROFF_MASK;
  iti_writew (tmp_val.all, &iti->pv_reg->frstdly.all);

  return (OK);
}  

int
pv_set_vscan (itifg_t *iti, u_char value) {

  pv_con1_t tmp_val;

  if (IS_PV_REV_A) {
    iti->errno = -EITI_NOSUPPORT;
    return (ERROR);
  }
  
  tmp_val.all = iti_readw (&iti->pv_reg->con1.all);
  tmp_val.b.vscan = value;
  iti_writew (tmp_val.all, &iti->pv_reg->con1.all);

  return (OK);
}

int
pv_set_vclkpol (itifg_t *iti, u_char value) {

  pv_con1_t tmp_val;

  if (IS_PV_REV_A) {
    iti->errno = -EITI_NOSUPPORT;
    return (ERROR);
  }
  
  tmp_val.all = iti_readw (&iti->pv_reg->con1.all);
  tmp_val.b.vclkpol = value;
  iti_writew (tmp_val.all, &iti->pv_reg->con1.all);

  return (OK);
}

int
pv_set_camsel (itifg_t *iti, u_char value) {

  pv_con1_t tmp_val;

  switch (value) {
  case PV_CAM_0:
  case PV_CAM_1:
  case PV_CAM_2:
  case PV_CAM_3:
    break;
  default:
  iti->errno = -EITI_NOSUPPORT;
    return (ERROR);
  }

  tmp_val.all = iti_readw (&iti->pv_reg->con1.all);
  tmp_val.b.camsel = value;
  iti_writew (tmp_val.all, &iti->pv_reg->con1.all);

  return (OK);
}

int
pv_set_strben (itifg_t *iti, u_char value) {

  pv_trigcon_t tmp_val;

  if (IS_PV_REV_A) {
    iti->errno = -EITI_NOSUPPORT;
    return (ERROR);
  }
  
  tmp_val.all = iti_readw (&iti->pv_reg->trigcon.all);
  tmp_val.b.strben = value;
  iti_writew (tmp_val.all, &iti->pv_reg->trigcon.all);

  return (OK);
}

int
pv_set_ccnt (itifg_t *iti, u_char value) {

  pv_p_clamp_t tmp_val;

  if (value > PV_MAX_CCNT) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }

  tmp_val.all = iti_readw (&iti->pv_reg->p_clamp.all);
  tmp_val.b.ccnt = value >> 1;
  iti_writew (tmp_val.all, &iti->pv_reg->p_clamp.all);

  return (OK);
}
  
int
pv_set_fdiv (itifg_t *iti, u_short value) {

  if (value < PV_FBKDIV_MIN || value > PV_FBKDIV_MAX) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }
  pv_write_pllreg (iti, PV_PLLREG0_ADDR, --value & PV_FBKDIV_MASK);

  return (OK);
}

int
pv_set_rdiv (itifg_t *iti, u_short value) {

  u_short tmp_val;

  if (value < PV_REFDIV_MIN || value > PV_REFDIV_MAX) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }

  tmp_val = (PV_PLLREG3_INIT & ~PV_REFDIV_MASK) | --value;
  pv_write_pllreg (iti, PV_PLLREG3_ADDR, tmp_val);

  return (OK);
}

int
pv_set_vcogain (itifg_t *iti, u_char value) {

  u_short tmp_val;

  if (value > PV_VCO_MASK) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }

  tmp_val = (PV_PLLREG4_INIT & ~PV_VCO_MASK) | value;
  pv_write_pllreg (iti, PV_PLLREG4_ADDR, tmp_val);

  return (OK);
}

int
pv_set_pda (itifg_t *iti, u_char value) {

  u_short tmp_val;

  value <<= PV_PDA_SHIFT;

  if (value > PV_PDA_MASK) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }

  tmp_val = (PV_PLLREG5_INIT & ~(PV_PDA_MASK << PV_PDA_SHIFT)) | value;
  pv_write_pllreg (iti, PV_PLLREG5_ADDR, tmp_val);

  return (OK);
}

int
pv_set_lcnt (itifg_t *iti, u_char value) {

  u_short tmp_val;

  if (value > PV_LCNT_MASK) {
    iti->errno = -EITI_RANGE;
    return (ERROR);
  }

  tmp_val = (PV_PLLREG6_INIT & ~PV_LCNT_MASK) | value;
  pv_write_pllreg (iti, PV_PLLREG6_ADDR, tmp_val);

  return (OK);
}

/*
 * Special Initialization Handler for PCVision Registers
 */

int
pv_init_adccon (itifg_t *iti) {

  if (pv_set_lutaddr (iti, PV_LUTADDR_CONT) < 0) return (ERROR);
  iti_writew (PV_RES_INIT, &iti->pv_reg->adcdata);

  return (OK);
}

int
pv_init_vsyncpw (itifg_t *iti) {

  iti_writew (PV_RES_INIT, &iti->pv_reg->vsyncpw.all);
  return (OK);
}
  
int
pv_init_strbdly (itifg_t *iti) {

  iti_writew (PV_RES_INIT, &iti->pv_reg->strbdly.all);
  return (OK);
}
  
int
pv_init_frstdly (itifg_t *iti) {

  iti_writew (PV_RES_INIT, &iti->pv_reg->frstdly.all);
  return (OK);
}
  
int
pv_init_con1 (itifg_t *iti) {

  iti_writew (PV_RES_INIT, &iti->pv_reg->con1.all);
  return (OK);
}

int
pv_init_con2 (itifg_t *iti) {

  iti_writew (PV_RES_INIT, &iti->pv_reg->con2.all);
  return (OK);
}

int
pv_init_trigcon (itifg_t *iti) {

  iti_writew (PV_RES_INIT, &iti->pv_reg->trigcon.all);
  return (OK);
}

int
pv_init_p_clamp (itifg_t *iti) {

  iti_writew (PV_RES_INIT, &iti->pv_reg->p_clamp.all);
  return (OK);
}

int
pv_init_pllregs (itifg_t *iti) {  

  if (pv_write_pllreg (iti, PV_PLLREG0_ADDR, PV_PLLREG0_INIT) < 0)
    return (ERROR);
  if (pv_write_pllreg (iti, PV_PLLREG1_ADDR, PV_PLLREG1_INIT) < 0)
    return (ERROR);
  if (pv_write_pllreg (iti, PV_PLLREG2_ADDR, PV_PLLREG2_INIT) < 0)
    return (ERROR);
  if (pv_write_pllreg (iti, PV_PLLREG3_ADDR, PV_PLLREG3_INIT) < 0)
    return (ERROR);
  if (pv_write_pllreg (iti, PV_PLLREG4_ADDR, PV_PLLREG4_INIT) < 0)
    return (ERROR);
  if (pv_write_pllreg (iti, PV_PLLREG5_ADDR, PV_PLLREG5_INIT) < 0)
    return (ERROR);
  if (pv_write_pllreg (iti, PV_PLLREG6_ADDR, PV_PLLREG6_INIT) < 0)
    return (ERROR);

  return (OK);
}
/* 
 * Miscellanous functions for PCVision
 */
 
/* trigger per software */
int
pv_sw_trig (itifg_t *iti) {

  iti_writew (PV_RES_INIT, &iti->pv_reg->sw_trig);
  return (OK);
}

/* linearize lookup table */
int
pv_lin_lut (itifg_t *iti) {

  int count;

  if (pv_set_lutaddr (iti, PV_LUTADDR_BASE) < 0) return (ERROR);

  for (count = PV_MIN_LUTDATA; count <= PV_MAX_LUTDATA; count++)
    iti_writew (count, &iti->pv_reg->lutdata);

  return (OK);
}

#if (DEBUG_ITIFG & 4)
/* make register dump */
void
pv_dump_regs (itifg_t *iti) {
  
  int count1;
  int count2;

  u_short *reg_ptr = (u_short*)iti->pv_reg;
  
  for (count1 = 0; count1 < PV_REG_SIZE; count1++, reg_ptr++)
    if (iti_strcmp (pv_reg_name[count1], "error  "))
      ITI_PRINT2A("    %s %#04x\n", pv_reg_name[count1], iti_readw (reg_ptr));

  for (count2 = 0; count2 < PV_PLL_SIZE; count1++, count2++)
    ITI_PRINT2A("    %s %#04x\n", pv_reg_name[count1],
		pv_read_pllreg (iti, count2));

  ITI_PRINT2A("    %s %#04x\n", pv_reg_name[count1++],
	      pv_get_nref(iti));
  ITI_PRINT2A ("    %s %#04x\n", pv_reg_name[count1],
	       pv_get_pref(iti));

}
#endif
