/*		 ITI-FG  Kernel module free public release
 *
 *		     Copyright (C) 1996-2000 GOM mbH.
 *	         Rebenring 33, 38106 Braunschweig, GERMANY
 *		            All rights reserved
 *
 * $Id: config_pcdig.c,v 7.0 2000/10/25 09:07:57 cvs Exp $
 *
 */

/* Feature test switches */

/* System headers */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <time.h>
#include <termios.h>

#include <sys/ioctl.h>
#include <sys/param.h>

#ifndef __NetBSD__
#include <getopt.h>
#endif

#ifdef linux
#  include <linux/limits.h>	/* PATH_MAX */
#endif

/* Local headers */

#include "itifgExt.h"
#include "iticonfig.h"

/* Macros */

#define FALLING	0	/* pixel clock falling edge */
#define RISING	1	/* pixel clock rising edge */

/* File scope variables */

/* External variables */

extern char *optarg;
extern int optind, opterr, optopt;

/* External functions */

/* Structures and unions */

/* Signal catching functions */

/* Functions */

/* Main */

int
main (int argc, char *argv[])
{
  int i = 0, option, error, board = -1; /* all boards */
  int depth = 8, taps = 1, edge = FALLING; /* 8bit - 1tap - falling edge */

  int boards, module[ITI_BOARDS_MAX];

  char device_name[PATH_MAX];
  char name_selection[PATH_MAX];

  snprintf (name_selection, PATH_MAX, "%sp%db%dt%c00.exo", DEF_CNFDIR,
	    depth / 8, taps, edge = FALLING ? 'n' : 'p');

  while (TRUE)
    {
      if ((option = getopt (argc, argv, "b:d:t:r?")) == -1) break;
      switch (option)
	{
	case 'b':
	  if (!optarg)
	    printf ("No board selected, using default (%d).\n", board);
	  else
	    {
	      board = atoi (optarg);
	      if (board < 0 || board >= ITI_BOARDS_MAX)
		{
		  printf ("Wrong board selected (%d).\n", board);
		  exit (EXIT_FAILURE);
		}
	    }
	  break;
	case 'd':
	  if (!optarg)
	    fprintf (stderr, "No image depth selected, using default (%d).\n",
		     depth);
	  else
	    {
	      depth = atoi (optarg);
	      switch (depth)
		{
		case 8:
		case 16:
		case 24:
		case 32:
		  break;
		default:
		  fprintf (stderr, "Wrong image depth selected (%s).\n",
			   optarg);
		  exit (EXIT_FAILURE);
		}
	    }
	  break;
	case 't':
	  if (!optarg)
	    fprintf (stderr, "No number of camera taps selected,"
		     "using default (%d).\n", taps);
	  else
	    {
	      taps = atoi (optarg);
	      switch (taps)
		{
		case 1:
		case 2:
		case 3:
		case 4:
		  break;
		default:
		  fprintf (stderr, "Wrong number of camera taps selected"
			   " (%s).\n", optarg);
		  exit (EXIT_FAILURE);
		}
	    }
	  break;
	case 'r':
	  edge = RISING;
	  break;
	case '?':
	default:
	  printf ("Usage: config_pcdig -b board (all) [0-%d]\n",
		  ITI_BOARDS_MAX - 1);
	  printf ("       a) fpga file is specified by parameters\n");
	  printf ("                    -d image depth (%d) [8|16|24|32]\n",
		  depth);
	  printf ("                    -t camera taps (%d) [1-4]\n", taps);
	  printf ("                    -r rising edge (falling)\n");
	  printf ("       b) fpga file is specified directly by name\n");
	  printf ("                    fpga file (%s)\n",
		  name_selection);
	  exit (EXIT_FAILURE);
	}
    }
  
  if (argc > optind)
    {
      /* specified by name */
      if (strcspn (argv[optind], "/") != strlen (argv[optind]))
	/* pathname and file is given */
	strcpy (name_selection, argv[optind]);
      else
	{
	  /* search into the default directory */
	  strcpy (name_selection, DEF_CNFDIR);
	  strcat (name_selection, argv[optind]);
	}
    }
  else
    {
      /* specified by parameters */
      snprintf (name_selection, PATH_MAX, "%sp%db%dt%c00.exo", DEF_CNFDIR,
		depth / 8, taps, edge = FALLING ? 'n' : 'p');
    }

  if ((error = read_procfs (NULL, NULL, &boards, module)))
    {
      fprintf (stderr, "pcdig: Error reading /proc/itifg. "
	       "Perhaps the driver is not loaded?\n");
      exit (EXIT_FAILURE);
    }

  if (board != -1)
    if (board < 0 || board >= boards || module[board] != ITI_VMPCD)
      {
	printf ("Wrong board selected (%d).\n", board);
	exit (EXIT_FAILURE);
      }

  for (i = board; i < boards; i++)
    {
      if (i < 0) i = 0;
      
      if (board == -1)
	if (module[i] != ITI_VMPCD)
	  continue;

      snprintf (device_name, PATH_MAX, "%s%d%s", ITI_NAME_PREFIX, i,
		ITI_NAME_DATA_POSTFIX);
      
      if ((error = convert_fpgafile (name_selection, device_name)) < 0)
	{
	  switch (error)
	    {
	    case -EITI_ARG:
	      fprintf (stderr, "pcdig: Wrong call argument (%s-%s).\n",
		       name_selection, device_name); 
	      exit (EXIT_FAILURE);
	      break;
	    case -EITI_FMT:
	      fprintf (stderr, "pcdig: Wrong data format (%s).\n",
		       name_selection); 
	      exit (EXIT_FAILURE);
	      break;
	    case -EITI_ENT:
	      fprintf (stderr, "pcdig: File not found (%s).\n",
		       name_selection);
	      exit (EXIT_FAILURE); 
	      break;
	    case -EITI_SYS:
	      fprintf (stderr, "pcdig: Error while system call (%s).\n",
		       strerror (errno)); 
	      exit (EXIT_FAILURE);
	      break;
	    default:
	      fprintf (stderr, "pcdig: Error not specified!\n"); 
	      exit (EXIT_FAILURE);
	    }
	}
      else
	{
	  fprintf (stderr, "pcdig: Downloaded to board %d.\n", i);
	}
      if (board != -1) break;
    }
  
  return (EXIT_SUCCESS);
}

